/*******************************************************************************


     Controlador de Temperatura de Ferro de Soldar

     Autor : Cludio Larios

     Data: 28/04/2015

     C/ pic 12f675

     Aplicao: Controlar temperatura de ferro de soldar, em 10 nveis,
     com indicao em leds retangulares (montados p/ formar um display de 7 seg).
     
     Temporizao para desligamento automtico aps +- 15 minutos.
     
     Obs. No use para controlar lampadas, mesmo incadescentes, pois gera
     tremulao desconfortavel.

     Este arquivo  parte integrante do blog 'LARIOS.TECNOLOGIA.WS'
     Uso didtico apenas.

*******************************************************************************/

#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, NOPUT, BROWNOUT
#use fast_io(a)
#zero_ram
#rom 0x2100 = {0,0}

#byte gpio   = 0x05
#byte trisio = 0x85
#byte tmr0   = 0x01

#bit  gate   = 0x85.5  //saida de disparo do triac
#bit  sw     = 0x05.3  //entrada da chave de escolha do nivel da temperatura
#bit  sw_off = 0x05.0  //entrada da chave de desligamento automatico temporizado
#bit  led_off = 0x85.0  //led indicador de temporizao
#bit  tris_sw_off = 0x85.0  //in-out

//variavis globais
int16 tempor;//temporizao de desligamento de 16 bits
int8 valor,valor_t,time;//variaveis de 8 bits
int1 s1,s2,dir,f5m,tr1,f_on,fr;//flags  gerais de 1 bit
//------------------------------------------------------------------------------
//tabela para acionamento dos leds
             //               -         +
             //gpio   //    543210    543210
const int8 array_t[]={    0b101001, 0b101001,// 0
                          0b101111, 0b111011,// 1
                          0b111001, 0b101010,// 2
                          0b101011, 0b101010,// 3
                          0b101111, 0b111000,// 4
                          0b101011, 0b101100,// 5
                          0b101001, 0b101100,// 6
                          0b101011, 0b111011,// 7
                          0b101001, 0b101000,// 8
                          0b101011, 0b101000,// 9
                          0b101101, 0b111000,// H
                      };

//------------------------------------------------------------------------------
//constantes
 const int8 t_ligado=15;//tempo que leds ficam ligados
 const int8 t_delay_incr=100;//tempo de delay para incrementos
 const int16 tempo_desl=230 * (15); //tempo p/ o desligamento automatico (min)

/*------------------------------------------------------------------------------
             INTERRUPO DO TIMER 0
------------------------------------------------------------------------------*/
#int_timer0
void isr_timer0(){  //a cada 100 us
static int8 vez,vez1,tmp;
   tmr0=156;
   if(++vez>24) {// a cada 2,5 ms
     vez=0;
     s1=!s1;
     tmp=t_ligado;
     f5m=1;
     trisio=0b111111;//desliga saidas
     if(fr){
        if(!sw_off) {f_on=1; } //testa boto de temporizao on/off
     }
     if(!s1){//testa qual grupo de leds sero acionados: + ou -
      gpio= 0x08;
      trisio=array_t[valor*2];
      if(valor_t){led_off=0;}//acende led de temporizao ligada
     }
     else{
     gpio= 0x1f;
     trisio=array_t[(valor*2)+1];
     }
     if(++vez1>10) vez1=0;
   }

   if(valor){
      if(vez1<=(valor)){ gate=!gate; }//inverte o pulso
      else{ gate=1;}//desliga pulsos
   }
     if(!--tmp){tmp=1;trisio=0b111111;}//desliga saidas
}

/*------------------------------------------------------------------------------
                     FIM DA INTERRUPO DO TIMER 0
------------------------------------------------------------------------------*/




/*==============================================================================
         ROTINA PRINCIPAL/ INICIALIZAO DE PORTS E REGISTRADORES
==============================================================================*/


void main() {

   setup_adc_ports(NO_ANALOGS|VSS_VDD);
   setup_adc(ADC_OFF);//desliga adc
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);//timer 0 sem prescaller
   setup_timer_1(T1_DISABLED);//timer 1 desligado
   setup_comparator(NC_NC_NC_NC);//desliga comparadores internos
   setup_vref(FALSE);//desliga v ref
   trisio= 0b111111;//trisio
   gpio=0b001000;//gpio
   enable_interrupts(INT_TIMER0);//liga interrupo do timer 0
   enable_interrupts(GLOBAL);//liga chave de interrupo global
   valor=read_eeprom(0); //recupera ajuste anterior
   valor_t=read_eeprom(1);//recupera estado do desligamento automtico
   s1=0;// zera flags
   s2=0;
   dir=0;
   fr=0;
   f_on=0;


/*==============================================================================
                    LAO PRINCIPAL DE REPETIO
==============================================================================*/
   for(;;){


       time=t_delay_incr;//determina a velocidade que ocorrer os incrementos
       while(time){
         fr=1;//flag de janela de teste do interruptor de desligamento 
         f5m=0;
         while(!f5m);
       --time;
       }
       fr=0;//bloqueia teste


       if(valor_t) {//testa se tempo maximo de desligamento foi alcanado
         if(++tempor==tempo_desl){ valor=0; tempor=0;}//desliga sada
       }

       if(!sw){//testa chave de incremento de temperatura
        if(++valor>10) valor=0;//incrementa valor e limita 
        tr1=1;//libera o flag de trava
        tempor=0;//zera tempo para nova contagem
       }
       else{//sw foi solta
        if(tr1){// grava eeprom uma unica vez por aperto de sw
         disable_interrupts(GLOBAL);
         write_eeprom(0,valor);//salva na eeprom a regulagem de temperatura
         tr1=0;//bloqueia flag de trava
         enable_interrupts(GLOBAL);
       }
      }



      if(f_on) { //liga/desliga temporizao

      if(valor_t) {
        disable_interrupts(GLOBAL);//desliga interrupo
        write_eeprom(1,0);//grava eeprom
        enable_interrupts(GLOBAL);
        valor_t=0;
        valor=read_eeprom(0); //religa
      } else {
        disable_interrupts(GLOBAL);//desliga interrupo
        write_eeprom(1,1);//grava eeprom
        enable_interrupts(GLOBAL);//religa interrupo
        valor_t=1;
        tempor=0;//inicia temporizao
        }

      time=t_delay_incr;//delay adicional
      while(time){
       f5m=0;
       while(!f5m);
       --time;
      }
      f_on=0;
      }

   }//for(;;)

}//main
